//
// FontMetrics.java - Minimal subset of java.awt.FontMetrics
//
// An class providing methods, allowing users of a given Font to determine
// characteristics of that font.
// Here we implement this by peeking directly inside class Font.
// 

package nyangau.miniawt;

public class FontMetrics
  {
  protected Font font;
  public FontMetrics(Font f) // Would normally be protected
    {
    font = f;
    }
  public Font getFont()
    {
    return font;
    }
  public int getAscent()
    {
    return font.ascent;
    }
  public int getMaxAscent()
    {
    return font.ascent;
    }
  public int getDescent()
    {
    return font.descent;
    }
  public int getMaxDescent()
    {
    return font.descent;
    }
  public int getLeading()
    {
    return font.leading;
    }
  public int getHeight()
    {
    return font.ascent+font.descent+font.leading;
    }
  public int charWidth(char c)
    {
    return font.advances[c];
    }
  public int charWidth(int c)
    {
    return font.advances[c];
    }
  public int bytesWidth(byte[] a, int off, int len)
    {
    int total = 0;
    for ( int i = 0; i < len; i++ )
      total += font.advances[a[off+i]];
    return total;
    }
  public int charsWidth(char[] a, int off, int len)
    {
    int total = 0;
    for ( int i = 0; i < len; i++ )
      total += font.advances[a[off+i]];
    return total;
    }
  public int stringWidth(String s)
    {
    int total = 0;
    for ( int i = 0; i < s.length(); i++ )
      total += font.advances[s.charAt(i)];
    return total;
    }
  public int[] getWidths()
    {
    int[] a = new int[0x100];
    for ( int i = 0; i < 0x100; i++ )
      a[i] = font.advances[i];
    return a;
    }
  public int getMaxAdvance()
    {
    int max = 0;
    for ( int i = 0; i < 0x100; i++ )
      if ( font.advances[i] > max )
        max = font.advances[i];
    return max;
    }
  }
