//
// Graphics.java - Minimal subset of java.awt.Graphics
//
// Provides a minimal set of drawing operations
//

package nyangau.miniawt;

import java.util.*;

public class Graphics
  {
  protected Image image;
  protected Color color;
  protected Font font = new Font(null, Font.PLAIN, 12);
  public Graphics(Image i) { image = i; } // Not normally public
  public Color getColor() { return color; }
  public void setColor(Color c) { color = c; }
  public Font getFont() { return font; }
  public void setFont(Font f) { font = f; }
  public FontMetrics getFontMetrics() { return new FontMetrics(font); }
  public FontMetrics getFontMetrics(Font f) { return new FontMetrics(f); }
  public void fillRect(int x, int y, int w, int h)
    {
    int y0 = Math.max(y, 0);
    int y1 = Math.min(y+h, image.height);
    int x0 = Math.max(x, 0);
    int x1 = Math.min(x+w, image.width );
    if ( x0 < x1 )
      for ( y = y0; y < y1; y++ )
        Arrays.fill(image.data, y*image.width+x0, y*image.width+x1, color.index);
    }
  protected void drawChar(char c, int x, int y)
    {
    int offset  = font.offsets [c];
    int advance = font.advances[c];
    y -= font.ascent;
    int y0 = Math.max(y, 0);
    int y1 = Math.min(y+font.ascent+font.descent, image.height);
    int x0 = Math.max(x, 0);
    int x1 = Math.min(x+advance, image.width);
    int r = (y0-y)*font.rw+offset;
    int d = y0*image.width;
    for ( int yi = y0; yi < y1; yi++, r+=font.rw, d+=image.width )
      {
      for ( int xi = x0; xi < x1; xi++ )
        if ( font.raster[r+(xi-x)] != 0 )
          image.data[d+xi] = color.index;
      }
    }
  public void drawString(String s, int x, int y)
    {
    for ( int i = 0; i < s.length(); i++ )
      {
      char c = s.charAt(i);
      drawChar(c, x, y);
      x += font.advances[c];
      }
    }
  }
