//
// PngEncoder.java - Serialise image in PNG format to OutputStream
//

package nyangau.miniawt;

import java.io.*;
import java.util.*;
import java.util.zip.*;

public class PngEncoder
  {
  protected Image image;
  protected void storeInt(byte[] buf, int offset, int i) 
    {
    buf[offset  ] = (byte) ((i>>>24) & 0xff);
    buf[offset+1] = (byte) ((i>>>16) & 0xff);
    buf[offset+2] = (byte) ((i>>> 8) & 0xff);
    buf[offset+3] = (byte) ( i       & 0xff);
    }
  public PngEncoder(Image i) { image = i; }
  protected void writeSignature(OutputStream os)
    throws IOException
    {
    byte[] sig = { (byte) -119, (byte) 'P' , (byte) 'N' , (byte) 'G' ,
                   (byte) '\r', (byte) '\n', (byte) 26  , (byte) '\n'  };
    os.write(sig);
    }
  protected void writeChunk(OutputStream os, String cn, byte[] cd, int cdlen)
    throws IOException
    {
    byte[] ch = new byte[8];
    storeInt(ch, 0, cdlen);
    ch[4] = (byte) cn.charAt(0);
    ch[5] = (byte) cn.charAt(1);
    ch[6] = (byte) cn.charAt(2);
    ch[7] = (byte) cn.charAt(3);
    os.write(ch);
    os.write(cd, 0, cdlen);
    CRC32 crc = new CRC32();
    crc.reset();
    crc.update(ch, 4, 4);
    crc.update(cd, 0, cdlen);
    byte[] cs = new byte[4];
    storeInt(cs, 0, (int) crc.getValue());
    os.write(cs);
    }
  protected void writeIHDR(OutputStream os)
    throws IOException
    {
    byte[] cd = new byte[13];
    storeInt(cd, 0, image.width);
    storeInt(cd, 4, image.height);
    cd[ 8] = 8; // bpp
    cd[ 9] = 3; // color type = palette index
    cd[10] = 0; // compression method = deflate
    cd[11] = 0; // filter method = none
    cd[12] = 0; // interlace = none
    writeChunk(os, "IHDR", cd, cd.length);
    }
  protected void writePLTE(OutputStream os)
    throws IOException
    {
    writeChunk(os, "PLTE", image.palette, 0x100*3);
    }
  protected void writeIDAT(OutputStream os)
    throws IOException
    {
    Deflater d = new Deflater(Deflater.BEST_COMPRESSION);
    ByteArrayOutputStream baos = new ByteArrayOutputStream(image.height*(1+image.width));
    DeflaterOutputStream dos = new DeflaterOutputStream(baos, d);
    byte[] filter = { 0 };
    for ( int y = 0; y < image.height; y++ )
      {
      dos.write(filter);
      dos.write(image.data, y*image.width, image.width);
      }
    dos.close();
    d.finish();
    writeChunk(os, "IDAT", baos.toByteArray(), baos.size());
    }
  protected void writeIEND(OutputStream os)
    throws IOException
    {
    byte[] cd = new byte[0];
    writeChunk(os, "IEND", cd, cd.length);
    }
  public void encode(OutputStream os)
    throws IOException
    {
    writeSignature(os);
    writeIHDR(os);
    writePLTE(os);
    writeIDAT(os);
    writeIEND(os);
    }
  }
